// -*- coding: utf-8 -*-

// Copyright (C) 2013 Stefan Besler <melongex@gmail.com>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


//! Initialize the plasmoid: Set global variables and listen to configuration changes
function initialize() {
  //  console.log("initializing...")
    plasmoid.setAspectRatioMode(IgnoreAspectRatio);
    plasmoid.globals = {} // Place to store global variables
    plasmoid.globals.host = "";
    plasmoid.globals.port = "";
    plasmoid.globals.apikey = "";

    plasmoid.globals.sickbeardHost = "";
    plasmoid.globals.sickbeardPort = "";
    plasmoid.globals.sickbeardApikey = "";
    plasmoid.globals.sickbeardDayOffset = 1;

    plasmoid.globals.status = {}
    plasmoid.globals.status.sizeleft = "0"
    plasmoid.globals.status.size = "0"
    plasmoid.globals.status.speed = "0"

    // Listen for config changes
    plasmoid.addEventListener('ConfigChanged', configChanged);
}

function statusProgressText() {
    return plasmoid.globals.status.sizeleft + "/" + plasmoid.globals.status.size;
}

function statusSpeedText() {
    var parts = plasmoid.globals.status.speed.split(" ");

    if(parts[0] === "0")
        return "";

    if(parts[0])
        return parts[0] + " kB/s";
    return plasmoid.globals.status.speed;
}

function statusStatusText() {
    if(plasmoid.globals.status.status === "Idle")
        return "";

    if(plasmoid.globals.status.paused === "False")
        return plasmoid.globals.status.status

    return "Paused"
}

//! This function is called whenever the config changes, or when the plasmoid is started
//! It will extract all important fields in the configuration dialog and fetch dota 2
//! data if the configuratino changed
function configChanged() {
    data.clear();

    polltimer.running = false;
    // Read configuration
    plasmoid.globals["host"] = "";//plasmoid.globals.host;
    plasmoid.globals["port"] = "";//plasmoid.globals.port;
    plasmoid.globals["apikey"] = "";//plasmoid.globals.apikey;

    plasmoid.globals["host"] = String(plasmoid.readConfig("host"))
    plasmoid.globals["port"] = String(plasmoid.readConfig("port"))
    plasmoid.globals["apikey"] = String(plasmoid.readConfig("apikey"))

    plasmoid.globals["sickbeardHost"] = String(plasmoid.readConfig("sickbeardHost"))
    plasmoid.globals["sickbeardPort"] = String(plasmoid.readConfig("sickbeardPort"))
    plasmoid.globals["sickbeardApikey"] = String(plasmoid.readConfig("sickbeardApikey"))
    plasmoid.globals["sickbeardDayOffset"] = 1;

    //headerText = plasmoid.globals.host + ":" + plasmoid.globals.port;
    headerText = plasmoid.globals.host + ":" + plasmoid.globals.port;

    if (!plasmoid.globals.host || !plasmoid.globals.port || !plasmoid.globals.apikey) {
        // Update overlay
        setTooltip("Error", "SABnzbp not configured.");
    }
    else
    {
        querySickbeardHistory();

        polltimer.interval = 15 * 60000;
        polltimer.running = true;
    }
}


//! This function fetches the players last matches from the dota2 webapi
function queryHistory(idStart) {
    //console.log("queryHistory, idStart="+idStart);
    var request = new XMLHttpRequest();
    request.onreadystatechange = function() {
        if (request.readyState === XMLHttpRequest.DONE && request.status === 200) {
            var document = request.responseXML.documentElement;

            var id=idStart;
            for(var i = 0; i < document.childNodes.length; i++) {
                var history = document.childNodes[i];
                if(history.tagName === "slots") {
                    for(var j = 0; j < history.childNodes.length; j++) {
                        var slots = history.childNodes[j];

                        if(slots.tagName === "slot") {
                            var dataServer = {
                                name: '',
                                date: '',
                                status: '',
                                size: '',
                                airdate: '',
                                ep_name: '',
                                episode: '',
                                season: '',
                                show_name: '',
                                tvdbid: '',
                                airs: '',
                                imageSource: '',
                                sizeleft: ''
                            };

                            for (var k = 0; k < slots.childNodes.length; k++){
                                var slot = slots.childNodes[k];

                                if(slot.childNodes && slot.childNodes.length > 0)
                                {
                                    if(slot.tagName === 'name'){
                                        dataServer.name = slot.childNodes[0].nodeValue;
                                    }
                                    else if(slot.tagName === 'completed'){
                                        dataServer.date = slot.childNodes[0].nodeValue;
                                    }
                                    else if(slot.tagName === 'status'){
                                        dataServer.status = slot.childNodes[0].nodeValue;
                                    }
                                    else if(slot.tagName === 'size'){
                                        dataServer.size = slot.childNodes[0].nodeValue;
                                    }
                                }
                            }

                            // Alles ist geparsed, jetzt schauen wir, ob wir unsere Daten mit Sickbeard abgleichen können
                            // Fertig -> Daten setzen
                            if(id < data.count)
                            {
                              //  console.log("Changing " + id + " data.count="+data.count);
                                data.set(id, dataServer);
                            }
                            else
                            {
                              //  console.log("Adding " + id + " data.count="+data.count);
                                data.append(dataServer);
                            }

                            syncWithSickbeard(id);
                            id = id + 1;
                        }
                    }
                }
            }

            //console.log("Resetting busy flag!!!!");
            plasmoid.setBusy(false);
            //console.log("data count is now " + data.count)
        }
    }
    var url = plasmoid.globals.host+":"+plasmoid.globals.port+"/sabnzbd/api?apikey="+plasmoid.globals.apikey+"&mode=history&output=xml";
    request.open('GET', url);
    request.send();
}

//! This function fetches the players last matches from the dota2 webapi
function queryInfos(idStart) {
    console.log("queryInfos, idStart="+idStart);

    var statusServer = {
        speed: '',
        sizeleft: '',
        size: '',
        limit: '',
        paused: '',
        status: '',
        timeleft: '',
        kbpersec: '',
    };
    var request = new XMLHttpRequest();
    request.onreadystatechange = function() {
        if (request.readyState === XMLHttpRequest.DONE && request.status === 200) {
            var document = request.responseXML.documentElement;
            for(var i = 0; i < document.childNodes.length; i++) {
                var queue = document.childNodes[i];
                if(queue.tagName === "speed") {
                    statusServer.speed = queue.childNodes[0].nodeValue.trim();
                }
                else if(queue.tagName === "size") {
                    statusServer.size = queue.childNodes[0].nodeValue.trim();
                }
                else if(queue.tagName === "sizeleft") {
                    statusServer.sizeleft = queue.childNodes[0].nodeValue.trim();
                }
                else if(queue.tagName === "limit") {
                    statusServer.limit = queue.childNodes[0].nodeValue.trim();
                }
                else if(queue.tagName === "paused") {
                    statusServer.paused = queue.childNodes[0].nodeValue.trim();
                }
                else if(queue.tagName === "status") {
                    statusServer.status = queue.childNodes[0].nodeValue.trim();
                }
                else if(queue.tagName === "timeleft") {
                    statusServer.timeleft = queue.childNodes[0].nodeValue.trim();
                }
                else if(queue.tagName === "kbpersec") {
                    statusServer.kbpersec = queue.childNodes[0].nodeValue.trim();
                }
                else if(queue.tagName === "slots") {
                    var id = idStart;
                    for(var j = 0; j < queue.childNodes.length; j++) {
                        var slots = queue.childNodes[j];
                        if(slots.tagName === "slot") {
                            var dataServer = {
                                name: '',
                                date: '',
                                status: '',
                                size: '',
                                airdate: '',
                                ep_name: '',
                                episode: '',
                                season: '',
                                show_name: '',
                                tvdbid: '',
                                airs: '',
                                imageSource: '',
                                sizeleft: ''
                            };

                            for (var k = 0; k < slots.childNodes.length; k++){
                                var slot = slots.childNodes[k];

                                if(slot.tagName === 'filename'){
                                    dataServer.name = slot.childNodes[0].nodeValue;
                                }
                                else if(slot.tagName === 'timeleft'){
                                    var parts = slot.childNodes[0].nodeValue.split(":");
                                    dataServer.date = ((new Date()).getTime() + 1000 * (60*60*parts[0] + 60*parseInt(parts[1]) + parseInt(parts[2]))) / 1000;
                                }
                                else if(slot.tagName === 'status'){
                                    dataServer.status = slot.childNodes[0].nodeValue;
                                }
                                else if(slot.tagName === 'mb'){
                                    dataServer.size = parseInt(slot.childNodes[0].nodeValue).toFixed(0) + " MB";
                                }
                                else if(slot.tagName === 'sizeleft'){

                                    dataServer.sizeleft = parseInt(slot.childNodes[0].nodeValue).toFixed(0) + " MB";
                                }
                            }

                            if(id < data.count)
                            {
                               // console.log("Changing " + id + " data.count="+data.count);
                                data.set(id, dataServer);
                            }
                            else
                            {
                              //  console.log("Adding " + id + " data.count="+data.count);
                                data.append(dataServer);
                            }

                            syncWithSickbeard(id);
                            id = id + 1;
                        }
                    }
                }
            }

            plasmoid.globals.status = statusServer;
            //statusProgress.text = statusProgressText();
            statusStatus.text = statusStatusText();
            statusSpeed.text = statusSpeedText();

            // Gui aktualisieren
            pauseButton.checked = (statusStatusText() === "Paused" ? true : false);
            queryHistory(id);

            // Abhängig vom Status wollen wir öfter aktualisieren
            if(statusStatusText() === "")
            {
                //console.log("Polling not so much now");
                polltimer.interval = 15 * 60000;
            }
            else
            {
                //console.log("Polling alot now");
                polltimer.interval = 0.5 * 60000;
            }
        }
    }
    var url = plasmoid.globals.host+":"+plasmoid.globals.port+"/sabnzbd/api?apikey="+plasmoid.globals.apikey+"&mode=queue&output=xml";
    request.open('GET', url);
    request.send();
  //  console.log(url)
}

function pad(value, length) {
    return (value.toString().length < length) ? pad("0"+value, length):value;
}

function querySickbeardHistory() {
    plasmoid.setBusy(true);

    //console.log("querySickbeardHistory");
    var request = new XMLHttpRequest();
    request.onreadystatechange = function() {

        if (request.readyState === XMLHttpRequest.DONE && request.status === 200) {
            var document = request.responseText;
            var jsonResponse = JSON.parse( document);

            sickbeardData.clear();
            for (var i in jsonResponse.data) {
                if(jsonResponse.data[i].status === "Snatched")
                {
                    var sickbeardDataServer = {
                        show_name: '',
                        season: '',
                        episode: '',
                        ep_name: '',
                        tvdbid: '',
                        imageSource: '',
                        resource: '',
                        status: ''
                    };

                    sickbeardDataServer.show_name = jsonResponse.data[i].show_name;
                    sickbeardDataServer.season = pad(jsonResponse.data[i].season,2);
                    sickbeardDataServer.episode = pad(jsonResponse.data[i].episode,2);
                    sickbeardDataServer.ep_name = "";
                    sickbeardDataServer.tvdbid = jsonResponse.data[i].tvdbid;
                    sickbeardDataServer.resource = jsonResponse.data[i].resource;
                    sickbeardDataServer.status = jsonResponse.data[i].status;
                    sickbeardDataServer.imageSource = plasmoid.globals.host+":"+plasmoid.globals.sickbeardPort+"/api/1234/?apikey="+plasmoid.globals.sickbeardApikey+"&cmd=show.getbanner&tvdbid="+sickbeardDataServer.tvdbid;

                   // console.log("Appending sickbeard history item:" + sickbeardDataServer.resource )
                    sickbeardData.append(sickbeardDataServer);
                }
            }

            //console.log("querySickbeardHistory - finished");
            queryComingEpisodes();
        }       
    }


    var url = plasmoid.globals.sickbeardHost+":"+plasmoid.globals.sickbeardPort+"/api/1234/?apikey="+plasmoid.globals.sickbeardApikey+"&cmd=history&limit=100";
    request.open('GET', url);
    // console.log(url);
    request.send();
}



function syncWithSickbeard(id) {

    //console.log("syncWithSickbeard, id="+id+"; sickbeardData.count=" +sickbeardData.count + "; data.count=" + data.count)
    for (var i=0; i<sickbeardData.count; ++i) {
        if(sickbeardData.get(i).status === "Snatched")
        {
            var resourceName = sickbeardData.get(i).resource.toLowerCase().replace(/\.[^/.]+$/, "");
            var dataName = data.get(id).name.toLowerCase().replace(/\.[^/.]+$/, "");

            //if(i < 2)
            //    console.log("resourceName= " + resourceName + "; dataName= "+dataName)
            // Testen ob wir die Daten zuordnen können
            if(resourceName.indexOf(dataName) >= 0 || dataName.indexOf(resourceName) >= 0 )
            {
                data.get(id).show_name = sickbeardData.get(i).show_name;
                data.get(id).season = pad(sickbeardData.get(i).season,2);
                data.get(id).episode = pad(sickbeardData.get(i).episode,2);
                data.get(id).ep_name = "";
                data.get(id).tvdbid = sickbeardData.get(i).tvdbid;
                data.get(id).imageSource = plasmoid.globals.host+":"+plasmoid.globals.sickbeardPort+"/api/1234/?apikey="+plasmoid.globals.sickbeardApikey+"&cmd=show.getbanner&tvdbid="+data.get(id).tvdbid;

                syncWithSickbeardMetaData(id)
                break;
            }
        }
    }
}


function syncWithSickbeardMetaData(id) {

   // console.log("Id:" + id + "dataCount:" + data.count + "sickbeardDataCount;"+sickbeardData.count)

    if(data.count < id)
        return;

    var tvdbid = data.get(id).tvdbid;
    var season = data.get(id).season;
    var episode = data.get(id).episode;

    var request = new XMLHttpRequest();
    request.onreadystatechange = function() {
        if (request.readyState === XMLHttpRequest.DONE && request.status === 200) {
            var document = request.responseText;
            var jsonResponse = JSON.parse(document);

            data.get(id).ep_name = jsonResponse.data.name;
        }
    }

    var url = plasmoid.globals.sickbeardHost+":"+plasmoid.globals.sickbeardPort+"/api/1234/?apikey="+plasmoid.globals.sickbeardApikey+"&cmd=episode&tvdbid="+tvdbid+"&season="+parseInt(season)+"&episode="+parseInt(episode);
    request.open('GET', url);
    request.send();
}

function queryComingEpisodes() {
  //  console.log("queryComingEpisodes ...");

    var id = 0;
    var request = new XMLHttpRequest();
    request.onreadystatechange = function() {
        if (request.readyState === XMLHttpRequest.DONE && request.status === 200) {
            var document = request.responseText;
            var jsonResponse = JSON.parse(document);
            var dataServer = {
                name: '',
                date: '',
                status: '',
                size: '',
                airdate: '',
                ep_name: '',
                episode: '',
                season: '',
                show_name: '',
                tvdbid: '',
                airs: '',
                imageSource: '',
                sizeleft: ''
            };

            for (var i in jsonResponse.data.missed) {
                dataServer.airdate = jsonResponse.data.missed[i].airdate;
                dataServer.ep_name = jsonResponse.data.missed[i].ep_name;
                dataServer.episode = pad(jsonResponse.data.missed[i].episode, 2);
                dataServer.season = pad(jsonResponse.data.missed[i].season,2);
                dataServer.show_name = jsonResponse.data.missed[i].show_name;
                dataServer.airs = jsonResponse.data.missed[i].airs;
                dataServer.tvdbid = jsonResponse.data.missed[i].tvdbid;
                dataServer.status = "Failed";
                dataServer.imageSource = plasmoid.globals.host+":"+plasmoid.globals.sickbeardPort+"/api/1234/?apikey="+plasmoid.globals.sickbeardApikey+"&cmd=show.getbanner&tvdbid="+dataServer.tvdbid;

                if(id < data.count)
                {
                   // console.log("Changing " + id + " data.count="+data.count);
                    data.set(id, dataServer);
                }
                else
                {
                  //  console.log("Adding " + id + " data.count="+data.count);
                    data.append(dataServer);
                }

                id = id + 1;
            }


            for (var i in jsonResponse.data.today) {
                dataServer.airdate = jsonResponse.data.today[i].airdate;
                dataServer.ep_name = jsonResponse.data.today[i].ep_name;
                dataServer.episode = pad(jsonResponse.data.today[i].episode, 2);
                dataServer.season = pad(jsonResponse.data.today[i].season,2);
                dataServer.show_name = jsonResponse.data.today[i].show_name;
                dataServer.airs = jsonResponse.data.today[i].airs;
                dataServer.tvdbid = jsonResponse.data.today[i].tvdbid;
                dataServer.status = "Soon";
                dataServer.imageSource = plasmoid.globals.host+":"+plasmoid.globals.sickbeardPort+"/api/1234/?apikey="+plasmoid.globals.sickbeardApikey+"&cmd=show.getbanner&tvdbid="+dataServer.tvdbid;

                if(id < data.count)
                {
                   // console.log("Changing " + id + " data.count="+data.count);
                    data.set(id, dataServer);
                }
                else
                {
                  //  console.log("Adding " + id + " data.count="+data.count);
                    data.append(dataServer);
                }

                id = id + 1;
            }

            for (i in jsonResponse.data.soon) {
                dataServer.airdate = jsonResponse.data.soon[i].airdate;
                dataServer.ep_name = jsonResponse.data.soon[i].ep_name;
                dataServer.episode = pad(jsonResponse.data.soon[i].episode,2);
                dataServer.season = pad(jsonResponse.data.soon[i].season,2);
                dataServer.show_name = jsonResponse.data.soon[i].show_name;
                dataServer.airs = jsonResponse.data.soon[i].airs;
                dataServer.tvdbid = jsonResponse.data.soon[i].tvdbid;
                dataServer.status = "Soon";
                dataServer.imageSource = plasmoid.globals.host+":"+plasmoid.globals.sickbeardPort+"/api/1234/?apikey="+plasmoid.globals.sickbeardApikey+"&cmd=show.getbanner&tvdbid="+dataServer.tvdbid

                if(id < data.count)
                {
                   // console.log("Changing " + id + " data.count="+data.count);
                    data.set(id, dataServer);
                }
                else
                {
                  //  console.log("Adding " + id + " data.count="+data.count);
                    data.append(dataServer);
                }

                id = id + 1;
            }

            // Wir sind fertig und können jetzt die Sabnzb History abfragen, wir beginnen mit ids data.count
            console.log("queryComingEpisodes finished");
            queryInfos(id);
        }
    }

    if(!plasmoid.globals.sickbeardHost || !plasmoid.globals.sickbeardPort)
    {
        console.log("No sickbeard binding");
        queryInfos(id);
    }
    else
    {
        var url = plasmoid.globals.sickbeardHost+":"+plasmoid.globals.sickbeardPort+"/api/1234/?apikey="+plasmoid.globals.sickbeardApikey+"&cmd=future&sort=date&type=missed|today|soon";
        request.open('GET', url);
        request.send();
        console.log(url);
    }
}

function hostName() {
    return plasmoid.globals.host;
}

function portName() {
    return plasmoid.globals.port;
}

function mbleft() {
    return plasmoid.globals.sizeleft;

}

function size() {
    return plasmoid.globals.size;
}

function speed() {
    return plasmoid.globals.speed;

}

function kbpersec() {
    return plasmoid.globals.kbpersec;

}

function timeleft() {
    return plasmoid.globals.timeleft;
}


//! How much time passed since this match?
function textWhen(startTime) {
    var date = new Date();
    var ts = Math.round((new Date()).getTime() / 1000);

    var minutes = parseFloat(ts - parseInt(startTime)) / 60;

    if(minutes > 0)
    {
        if(minutes < 60)
            return Math.round(minutes)+" m";

        var hours = minutes / 60;
        if(hours < 24)
            return Math.round(hours)+" h";

        var days = Math.round(hours / 24);
        var dayString = "d";
        if(days === 1)
            dayString = "d";

        return days + " " + dayString;
    }
    return "";
   /* else
    {
        minutes = minutes * -1;
        if(minutes < 60)
            return "In " + Math.round(minutes)+" m";

        hours = minutes / 60;
        if(hours < 24)
            return "In " +Math.round(hours)+" h";

        var days = Math.round(hours / 24);
        var dayString = "d";
        if(days === 1)
            dayString = "d";

        return "In " +days + " " + dayString;
    }*/
}

function sickbeardWhen(airdate) {
    var now = new Date().getTime();
    var parts = airdate.split("-");

    // Achtung, das Monat in Javascript beginnt bei 0 :-D
    var date = new Date(parseFloat(parts[0]), parseFloat(parts[1]-1), parseFloat(parts[2]));
    var days = Math.ceil((date - now) / 1000 / 60 / 60 / 24) + plasmoid.globals.sickbeardDayOffset;

    var dayString = "days";
    if(days === 1)
        dayString = "day";

    if(days < 0)
        return days + " "+dayString+" ago";
    else if(days == 0)
        return "Today";
    else if(days == 1)
        return "Tomorrow";

    return "In " + days + " "+dayString;
}

function subText(date, airdate) {
    if(date)
        return textWhen(date);
    return sickbeardWhen(airdate);
}

function subTextAlternative(size, sizeleft, airs) {

    if(sizeleft)
        return "-" + parseFloat(sizeleft).toFixed(0) + " MB";

    if(size)
        return size;
    return airs;
}

function statusColor(status) {
    if(status === "Completed")
        //return "#15fdfdfd";
        return theme.buttonBackgroundColor;
    else if(status === "Failed")
        return "#15fca500";

    return "green";
}

function statusIcon(status, startTime) {
    var date = new Date();
    var ts = Math.round((new Date()).getTime() / 1000);
    var days = parseFloat(ts - parseInt(startTime)) / 60 / 60 / 24;

    if(Math.round(days) < 2 && status === "Completed") {
        return "document-new";
    }
    else if(status === "Completed") {
        return "dialog-ok";
    }
    else if(status === "Queued") {
        return "svn-update";
    }
    else if(status === "Downloading") {
        return "download";
    }
    else if(status === "Extracting") {
        return "ark";
    }
    else if(status === "Verifying") {
        return "document-edit-verify";
    }
    else if(status === "Failed") {
        return "dialog-cancel";
    }
    else if(status === "Soon") {
        return "tools-wizard";
    }


    console.log("Unknown state: " + status);
    return "tools-wizard";
}

function enablePause(on) {
  //  console.log("enablePause: " + on)
    var request = new XMLHttpRequest();
    request.onreadystatechange = function() {
        if (request.readyState === XMLHttpRequest.DONE && request.status === 200) {
            if(request.responseText.trim() === "ok")
            {
                querySickbeardHistory();

                if(pauseButton.checked !== on)
                    pauseButton.checked = on;
            }
        }
    }

    var url = plasmoid.globals.host+":"+plasmoid.globals.port+"/sabnzbd/api?apikey="+plasmoid.globals.apikey+"&mode=";
    if(on)
        url += "pause";
    else
        url += "resume";

    request.open('GET', url);
    request.send();
}

