// -*- coding: utf-8 -*-

// Copyright (C) 2013 Stefan Besler <melongex@gmail.com>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

import QtQuick 1.0
import org.kde.plasma.components 0.1 as PlasmaComponents
import org.kde.plasma.graphicswidgets 0.1 as PlasmaWidgets
import org.kde.plasma.core 0.1 as PlasmaCore
import "plasmapackage:/code/sabnzbp.js" as SNP

Rectangle {
    id: wrapper
    color: "transparent"

    // Setup the plasmoid
    property int minimumWidth: 200
    property int minimumHeight: 250
    property int headerHeight: 40
    property int footerHeight: 25
    property int headerMargin: 6
    property variant status
    property string headerText: ""

    ListModel {
        id: data
    }

    ListModel {
        id: sickbeardData
    }

    ListModel {
        id: comingData
    }

    Component {
        id: historyDelegate
        //Item {
        Item {
            id: historyItem
            width: wrapper.width
            height: wrapper.headerHeight
            state: "WHEN"

            Column {property int headerHeight: 40

                // Hauptansicht
                Rectangle {
                    id: item
                    width: historyItem.width
                    height: historyItem.height - wrapper.headerMargin
                    color: "transparent"
                    Row {
                        anchors {
                            fill: parent
                            centerIn: parent
                        }

                        ProgressBar {
                            id:progress
                            width: historyItem.width
                            height: wrapper.headerHeight - wrapper.headerMargin
                           // color: SNP.statusColor(status)
                            icon: SNP.statusIcon(status, date)
                            value: 100
                            text: {
                                if(show_name)
                                    return show_name + " - " + "S"+season + "E"+episode + " - " + ep_name;
                                return name;
                            }
                            when: SNP.textWhen(date)
                            imageSrc: imageSource
                            state: status
                        }
                    }
                }
            }


            states: [
                     State {
                         name: "WHEN"
                         when: (historyItem.state == "WHEN")
                         PropertyChanges { target: progress; when: SNP.subText(date, airdate) }
                     },
                     State {
                         name: "SIZE"
                         when: (historyItem.state == "SIZE")
                         PropertyChanges { target: progress; when: SNP.subTextAlternative(size, sizeleft, airdate) }
                     }
                 ]

            // Elemente anklicken und highlighten
            MouseArea {
                anchors.right: parent.right
                width: 100
                height: wrapper.headerHeight
                onEntered: { listView.currentIndex = index; historyItem.state = "SIZE"; }
                onExited: { historyItem.state = "WHEN"; }

                hoverEnabled: true
            }
        }
    }

    Component.onCompleted: {
        SNP.initialize();
    }

    Column {
        anchors.fill: parent
        anchors.centerIn: parent

        Rectangle {
            id: header
            width: listView.width
            height: 18
            color: "transparent"
            Row {
                anchors.centerIn: parent
                PlasmaComponents.Label {
                    id: plasmoidHeader
                    verticalAlignment: Qt.AlignVCenter
                    horizontalAlignment: Qt.AlignHCenter
                    font.capitalization: Font.AllLowercase
                    font.bold: true
                    font.weight: Font.DemiBold
                    text: { return wrapper.headerText;}
                    }
             }
        }

        ListView {
            id: listView
            width: wrapper.width
            height: wrapper.height - header.height - wrapper.footerHeight - wrapper.headerMargin

            focus: true
            clip: true
            model: data
            delegate: historyDelegate
            highlightFollowsCurrentItem: false
         //   highlight: highlightDelegate
        }

        PlasmaCore.Theme {
            id: theme
        }

        Timer {
            id: polltimer
            running: false
            repeat: true
            onTriggered: { SNP.querySickbeardHistory(); }
        }

        Rectangle {
            color: "transparent";
            width: wrapper.width;
            height:wrapper.headerMargin;
        }


        Row {
            spacing: 5
            PlasmaComponents.Button {
                id: pauseButton
                checkable: true
                text: "Pause"
               // iconSource: "appointment-recurring"
                onClicked: { SNP.enablePause(pauseButton.checked);  }
            }

            PlasmaComponents.Button {
                id: refreshButton
                checkable: false
                //iconSource: "media-playback-pause"
                text: "Refresh"
                onClicked: { SNP.querySickbeardHistory();  }
            }

            PlasmaComponents.Label {
                horizontalAlignment: Qt.AlignHCenter
                //  verticalAlignment: Qt.AlignBottom
                width: (wrapper.width - pauseButton.width - refreshButton.width) / 2
                height: wrapper.footerHeight
                // height: wrapper.footerHeight
                id: statusStatus
                //text: ""
            }

            PlasmaComponents.Label {
                horizontalAlignment: Qt.AlignHCenter
                //  verticalAlignment: Qt.AlignBottom
                width: statusStatus.width
                height: wrapper.footerHeight
                //  height: wrapper.footerHeight
                id: statusSpeed
                //text: ""
            }
        }
    }
}

